//
//  Bluetooth4Whm.m
//  WHM08
//
//  Created by 富樫 豊彦 on 2013/11/10.
//  Copyright (c) 2013年 富樫 豊彦. All rights reserved.
//

#import <CoreBluetooth/CoreBluetooth.h>
#import "Bluetooth4Whm.h"


@interface Bluetooth4Whm () <CBCentralManagerDelegate, CBPeripheralDelegate>
@property (strong, nonatomic) CBCentralManager      *centralManager;
@property (strong, nonatomic) CBPeripheral          *discoveredPeripheral;
@end


@implementation Bluetooth4Whm


- (id)init
{
    self = [super init];
    if (self) {
        self.centralManager = [[CBCentralManager alloc] initWithDelegate:self queue:nil];
    }
    NSLog(@"BT Init");
    return self;
}


#pragma mark - Bluetooth-LE central


- (void)centralManagerDidUpdateState:(CBCentralManager *)central
{
    NSLog(@"BT update %ld", central.state);
    if (central.state != CBCentralManagerStatePoweredOn) {
        return;
    }
    [self scan];
}


- (void)scan
{
    [self.centralManager scanForPeripheralsWithServices:@[[CBUUID UUIDWithString:SERVICE_UUID]] options:nil];
    //options:@{ CBCentralManagerScanOptionAllowDuplicatesKey : @YES }];
    NSLog(@"Scanning...");
}


- (void)centralManager:(CBCentralManager *)central didDiscoverPeripheral:(CBPeripheral *)peripheral advertisementData:(NSDictionary *)advertisementData RSSI:(NSNumber *)RSSI
{
    NSLog(@"Discovered %@ at %@db", peripheral.name, RSSI);
    if (self.discoveredPeripheral != peripheral) {
        self.discoveredPeripheral = peripheral;
        NSLog(@"Connecting to peripheral %@", peripheral);
        [self.centralManager connectPeripheral:peripheral options:nil];
    }
}


- (void)centralManager:(CBCentralManager *)central didFailToConnectPeripheral:(CBPeripheral *)peripheral error:(NSError *)error
{
    NSLog(@"Failed to connect to %@. (%@)", peripheral, [error localizedDescription]);
    [self cleanup];
}


- (void)centralManager:(CBCentralManager *)central didConnectPeripheral:(CBPeripheral *)peripheral
{
    NSLog(@"Peripheral Connected");
    [self.centralManager stopScan];
    peripheral.delegate = self;
    [peripheral discoverServices:@[[CBUUID UUIDWithString:SERVICE_UUID]]];
    self.valid = 1;
}


- (void)peripheral:(CBPeripheral *)peripheral didDiscoverServices:(NSError *)error
{
    if (error) {
        NSLog(@"Error discovering services: %@", [error localizedDescription]);
        [self cleanup];
        return;
    }
    for (CBService *service in peripheral.services) {
        [peripheral discoverCharacteristics:@[[CBUUID UUIDWithString:CHARACTERISTIC_UUID0]] forService:service];
        [peripheral discoverCharacteristics:@[[CBUUID UUIDWithString:CHARACTERISTIC_UUID1]] forService:service];
        [peripheral discoverCharacteristics:@[[CBUUID UUIDWithString:CHARACTERISTIC_UUID2]] forService:service];
        [peripheral discoverCharacteristics:@[[CBUUID UUIDWithString:CHARACTERISTIC_UUID3]] forService:service];
        [peripheral discoverCharacteristics:@[[CBUUID UUIDWithString:CHARACTERISTIC_UUID4]] forService:service];
        [peripheral discoverCharacteristics:@[[CBUUID UUIDWithString:CHARACTERISTIC_UUID5]] forService:service];
        [peripheral discoverCharacteristics:@[[CBUUID UUIDWithString:CHARACTERISTIC_UUID6]] forService:service];
        [peripheral discoverCharacteristics:@[[CBUUID UUIDWithString:CHARACTERISTIC_UUID7]] forService:service];
    }
}


- (void)peripheral:(CBPeripheral *)peripheral didDiscoverCharacteristicsForService:(CBService *)service error:(NSError *)error
{
    if (error) {
        NSLog(@"Error discovering characteristics: set notify: %@", [error localizedDescription]);
        [self cleanup];
        return;
    }
    for (CBCharacteristic *characteristic in service.characteristics) {
        if ([characteristic.UUID isEqual:[CBUUID UUIDWithString:CHARACTERISTIC_UUID0]]) {   // Date time
            struct {
                short yyyy;
                char mm;
                char dd;
                char hh;
                char mi;
                char ss;
            } val;
            struct tm tm;
            time_t  t;
            
            t = time(&t);
            localtime_r(&t, &tm);
            val.yyyy = tm.tm_year + 1900;
            val.mm   = tm.tm_mon + 1;
            val.dd   = tm.tm_mday;
            val.hh   = tm.tm_hour;
            val.mi   = tm.tm_min;
            val.ss   = tm.tm_sec;
            NSData *val1 = [[NSData alloc] initWithBytes:&val length:sizeof(val)];
            [peripheral  writeValue:val1 forCharacteristic:characteristic type:CBCharacteristicWriteWithResponse];
            NSLog(@"Send date time %@", characteristic);
        } else {
            [peripheral setNotifyValue:YES forCharacteristic:characteristic];
            NSLog(@"Set notify %@", characteristic);
        }
    }
}


- (void)peripheral:(CBPeripheral *)peripheral didUpdateValueForCharacteristic:(CBCharacteristic *)characteristic error:(NSError *)error
{
    int     channelNumber;
    float   power;
    float   current;
    float   voltage;
    float   frequency;
    float   energyOfMonth;
    
    if (error) {
        NSLog(@"Error discovering characteristics: update value: %@", [error localizedDescription]);
        return;
    }

    channelNumber = 0;
    if ([characteristic.UUID isEqual:[CBUUID UUIDWithString:CHARACTERISTIC_UUID0]]) {
        int yyyy;
        int mm;
        int dd;
        int hh;
        int mi;
        int ss;
        
        yyyy = *((short *)[characteristic.value bytes]);        // Byte order is Little endian
        mm   = *((char *)[characteristic.value bytes] + 2);
        dd   = *((char *)[characteristic.value bytes] + 3);
        hh   = *((char *)[characteristic.value bytes] + 4);
        mi   = *((char *)[characteristic.value bytes] + 5);
        ss   = *((char *)[characteristic.value bytes] + 6);
        self.date = yyyy * 10000 + mm * 100 + dd;
        self.time = hh * 10000 + mi * 100 + ss;
        NSLog(@"Date=%ld Time=%ld", self.date, self.time);
    } else if ([characteristic.UUID isEqual:[CBUUID UUIDWithString:CHARACTERISTIC_UUID1]]) {
        channelNumber = 1;
    } else if ([characteristic.UUID isEqual:[CBUUID UUIDWithString:CHARACTERISTIC_UUID2]]) {
        channelNumber = 2;
    } else if ([characteristic.UUID isEqual:[CBUUID UUIDWithString:CHARACTERISTIC_UUID3]]) {
        channelNumber = 3;
    } else if ([characteristic.UUID isEqual:[CBUUID UUIDWithString:CHARACTERISTIC_UUID4]]) {
        channelNumber = 4;
    } else if ([characteristic.UUID isEqual:[CBUUID UUIDWithString:CHARACTERISTIC_UUID5]]) {
        channelNumber = 5;
    } else if ([characteristic.UUID isEqual:[CBUUID UUIDWithString:CHARACTERISTIC_UUID6]]) {
        channelNumber = 6;
    } else if ([characteristic.UUID isEqual:[CBUUID UUIDWithString:CHARACTERISTIC_UUID7]]) {
        channelNumber = 7;
    }
    if (channelNumber > 0) {
        power         = *((float *)[characteristic.value bytes]);
        current       = *((float *)[characteristic.value bytes] + 1);
        voltage       = *((float *)[characteristic.value bytes] + 2);
        frequency     = *((float *)[characteristic.value bytes] + 3);
        energyOfMonth = *((float *)[characteristic.value bytes] + 4) / 1000.0;
        
        // Update the CoreData
        NSManagedObjectContext *context = [self.fetchedResultsController managedObjectContext];
        NSFetchRequest *fetchRequest = [[NSFetchRequest alloc] init];
        NSEntityDescription *entity = [NSEntityDescription entityForName:@"Event" inManagedObjectContext:context];
        [fetchRequest setEntity:entity];
        NSPredicate *predicate = [NSPredicate predicateWithFormat:@"chNumber = %d",channelNumber];
        [fetchRequest setPredicate:predicate];
        NSError *error;
        NSArray *fetchedObjects = [context executeFetchRequest:fetchRequest error:&error];
        if (fetchedObjects == nil) {
            NSLog(@"Unresolved error %@, %@", error, [error userInfo]);
            abort();
        }
        [fetchedObjects setValue:[[NSNumber alloc] initWithInt:channelNumber]      forKey:@"chNumber"];
        [fetchedObjects setValue:[[NSNumber alloc] initWithFloat:power]            forKey:@"power"];
        [fetchedObjects setValue:[[NSNumber alloc] initWithFloat:current]          forKey:@"current"];
        [fetchedObjects setValue:[[NSNumber alloc] initWithFloat:voltage]          forKey:@"voltage"];
        [fetchedObjects setValue:[[NSNumber alloc] initWithFloat:frequency]        forKey:@"frequency"];
        [fetchedObjects setValue:[[NSNumber alloc] initWithFloat:energyOfMonth]    forKey:@"energy"];
    }
}


- (void)centralManager:(CBCentralManager *)central didDisconnectPeripheral:(CBPeripheral *)peripheral error:(NSError *)error
{
    NSLog(@"Peripheral Disconnected");
    self.discoveredPeripheral = nil;
    [self scan];
    self.valid = 0;
}


- (void)cleanup
{
    if (self.discoveredPeripheral.state == CBPeripheralStateConnected) {
        [self.centralManager cancelPeripheralConnection:self.discoveredPeripheral];
        NSLog(@"BT Cleanup");
    }
    self.valid = 0;
}

@end
